package legacy;

import org.apache.commons.codec.binary.Base64;

import java.io.*;

/**
 * CSVへの変換元であるファイルをラップしたクラスです。
 *
 */
public class Source {
    private final Parameter parameter;
    private final File file;
    private final long size;
    private final long lastModified;

    private Source(Parameter parameter, File file, long size, long lastModified) {
        this.parameter = parameter;
        this.file = file;
        this.size = size;
        this.lastModified = lastModified;
    }

    /**
     * ファイルが空でなければ{@literal true}を返します。
     * 
     * @return
     */
    public boolean isNotEmpty() {
        return size > 0L;
    }

    /**
     * ファイル名の末尾が指定された文字列に合致すれば{@literal true}を返します。
     * 
     * @param ext
     * @return
     */
    public boolean suffix(String ext) {
        return file.getName().endsWith(ext);
    }

    /**
     * ファイルの更新時刻が{@link Parameter#basetime basetime}より新しければ{@literal true}を返します。
     * 
     * @return
     */
    public boolean isNewer() {
        return lastModified > parameter.basetime;
    }

    /**
     * ファイルの更新時刻・srcDirからの相対パス・ファイルの内容をBase64エンコードしたものを
     * ","で区切ったCSV形式にして返します。
     * 
     * @return
     * @throws IOException
     */
    public String toCsv() throws IOException {
        StringBuilder buf = new StringBuilder();
        buf.append(String.valueOf(lastModified));
        buf.append(',');
        buf.append(relativePath(parameter.srcDir, file));
        buf.append(',');
        buf.append(toBase64(file));
        return buf.toString();
    }

    static String toBase64(File file) throws IOException {
        InputStream in = new FileInputStream(file);
        try {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int i;
            byte[] b = new byte[1024];
            while (-1 != (i = in.read(b))) {
                baos.write(b, 0, i);
            }
            //commons-codec
            return Base64.encodeBase64String(baos.toByteArray());
        } finally {
            in.close();
        }
    }

    static String relativePath(File srcDir, File file) {
        return file.getAbsolutePath()
                .substring((srcDir.getAbsolutePath() + File.separator).length());
    }

    public static class Builder {
        private final Parameter parameter;

        public Builder(Parameter parameter) {
            this.parameter = parameter;
        }

        public Source build(File path) {
            long size = path.length();
            long lastModified = path.lastModified();
            return new Source(parameter, path, size, lastModified);
        }
    }
}
