package modern;

import java.io.IOException;
import java.io.UncheckedIOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Base64;
import java.util.StringJoiner;
import java.util.function.Predicate;

/**
 * CSVへの変換元であるファイルをラップしたクラスです。
 *
 */
public class Source {
    private final Parameter parameter;
    private final Path file;
    private final long size;
    private final long lastModified;

    private Source(Parameter parameter, Path file, long size, long lastModified) {
        this.parameter = parameter;
        this.file = file;
        this.size = size;
        this.lastModified = lastModified;
    }

    /**
     * ファイルが空でなければ{@literal true}を返します。
     * 
     * @return
     */
    public boolean isNotEmpty() {
        return size > 0L;
    }

    /**
     * ファイル名の末尾が指定された文字列に合致すれば{@literal true}を返す{@link Predicate}を構築します。
     * 
     * @param ext
     * @return
     */
    public static Predicate<Source> suffix(String ext) {
        return a -> a.file.getFileName().toString().endsWith(ext);
    }

    /**
     * ファイルの更新時刻が{@link Parameter#basetime basetime}より新しければ{@literal true}を返します。
     * 
     * @return
     */
    public boolean isNewer() {
        return lastModified >= parameter.basetime;
    }

    /**
     * ファイルの更新時刻・srcDirからの相対パス・ファイルの内容をBase64エンコードしたものを
     * ","で区切ったCSV形式にして返します。
     * 
     * @return
     * @throws IOException
     */
    public String toCsv() throws IOException {
        return new StringJoiner(",")
                .add(String.valueOf(lastModified))
                .add(relativePath(parameter.srcDir, file))
                .add(toBase64(file))
                .toString();
    }

    static String toBase64(Path file) throws IOException {
        return Base64.getEncoder().encodeToString(Files.readAllBytes(file));
    }

    static String relativePath(Path srcDir, Path file) {
        return srcDir.relativize(file).toString();
    }

    public static class Builder {
        private final Parameter parameter;
        public Builder(Parameter parameter) {
            this.parameter = parameter;
        }
        public Source build(Path path) {
            try {
                long size = Files.size(path);
                long lastModified = Files.getLastModifiedTime(path).toMillis();
                return new Source(parameter, path, size, lastModified);
            } catch (IOException e) {
                throw new UncheckedIOException(e);
            }
        }
    }
}
